"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createQueryWithHistoryAutocompleter = void 0;
const query_history_autocompleter_1 = require("./query-history-autocompleter");
const query_autocompleter_1 = require("./query-autocompleter");
const compass_components_1 = require("@mongodb-js/compass-components");
const createQueryWithHistoryAutocompleter = ({ savedQueries, options = {}, queryProperty, onApply, theme, }) => {
    const queryHistoryAutocompleter = (0, query_history_autocompleter_1.createQueryHistoryAutocompleter)({
        savedQueries,
        onApply,
        queryProperty,
        theme,
    });
    const originalQueryAutocompleter = (0, query_autocompleter_1.createQueryAutocompleter)(options);
    const historySection = {
        name: 'Query History',
        header: renderDottedLine,
    };
    return async function fullCompletions(context) {
        if (context.state.doc.toString() === '{}') {
            return queryHistoryAutocompleter(context);
        }
        const combinedOptions = [];
        // completions assigned to a section appear below ones that are not assigned
        const baseCompletions = await originalQueryAutocompleter(context);
        const historyCompletions = await queryHistoryAutocompleter(context);
        if (baseCompletions) {
            combinedOptions.push(...baseCompletions.options.map((option) => ({
                ...option,
            })));
        }
        if (historyCompletions) {
            combinedOptions.push(...historyCompletions.options.map((option) => ({
                ...option,
                section: historySection,
            })));
        }
        return {
            from: Math.min(historyCompletions?.from ?? context.pos, baseCompletions?.from ?? context.pos),
            options: combinedOptions,
        };
    };
};
exports.createQueryWithHistoryAutocompleter = createQueryWithHistoryAutocompleter;
const sectionHeaderStyles = (0, compass_components_1.css)({
    display: 'list-item',
    borderBottom: '1px dashed #ccc',
    margin: `5px 0`,
});
function renderDottedLine() {
    const header = document.createElement('div');
    header.className = sectionHeaderStyles;
    return header;
}
//# sourceMappingURL=query-autocompleter-with-history.js.map