"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAncestryOfToken = exports.getTokenText = exports.getPropertyNameFromPropertyToken = exports.isTokenEqual = exports.removeQuotes = exports.parents = exports.padLines = exports.aggLink = exports.resolveTokenAtCursor = exports.completeWordsInString = void 0;
const autocomplete_1 = require("@codemirror/autocomplete");
const language_1 = require("@codemirror/language");
const compass_components_1 = require("@mongodb-js/compass-components");
exports.completeWordsInString = (0, autocomplete_1.ifIn)(['String'], autocomplete_1.completeAnyWord);
function resolveTokenAtCursor(context) {
    return (0, language_1.syntaxTree)(context.state).resolveInner(context.pos, -1);
}
exports.resolveTokenAtCursor = resolveTokenAtCursor;
function aggLink(op, { utmSource, utmMedium }) {
    op = op.replace(/^\$/, '');
    const href = (0, compass_components_1.urlWithUtmParams)(`https://www.mongodb.com/docs/manual/reference/operator/aggregation/${op}/`, { utmSource, utmMedium });
    return `<a target="_blank" href="${href}">$${op}</a>`;
}
exports.aggLink = aggLink;
function padLines(str, pad = '  ') {
    return str
        .split('\n')
        .map((line) => `${pad}${line}`)
        .join('\n');
}
exports.padLines = padLines;
function* parents(token) {
    let parent = token;
    while ((parent = parent.parent)) {
        yield parent;
    }
}
exports.parents = parents;
function removeQuotes(str) {
    return str.replace(/(^('|")|('|")$)/g, '');
}
exports.removeQuotes = removeQuotes;
// lezer tokens are immutable, we check position in syntax tree to make sure we
// are looking at the same token
function isTokenEqual(a, b) {
    return a.from === b.from && a.to === b.to;
}
exports.isTokenEqual = isTokenEqual;
function getPropertyNameFromPropertyToken(editorState, propertyToken) {
    if (!propertyToken.firstChild) {
        return '';
    }
    return removeQuotes(getTokenText(editorState, propertyToken.firstChild));
}
exports.getPropertyNameFromPropertyToken = getPropertyNameFromPropertyToken;
function getTokenText(editorState, token) {
    return editorState.sliceDoc(token.from, token.to);
}
exports.getTokenText = getTokenText;
/**
 * Returns the list of possible ancestors of the token.
 */
const getAncestorList = (node, doc) => {
    const ancestors = [];
    if (!node?.parent) {
        return ancestors;
    }
    // In order to find the index of an node in an array, we have to check for the parent.
    // And then use siblings to find the index. We can't use the node.parent as it does not
    // contain all the childern and reference to the current node.
    if (['ArrayExpression', 'Array'].includes(node.parent.name)) {
        let prevSibling = node.prevSibling;
        let index = 0;
        while (prevSibling) {
            if (![':', ',', '['].includes(prevSibling.name)) {
                index++;
            }
            prevSibling = prevSibling.prevSibling;
        }
        ancestors.push(`[${index}]`);
    }
    if (node.name === 'Property' && node.firstChild) {
        const { from, to } = node.firstChild;
        const ancestor = doc.slice(from, to).replace(/"/g, '').replace(/'/g, '');
        ancestors.push(ancestor);
    }
    return ancestors.concat(getAncestorList(node.parent, doc));
};
/**
 * Walks the syntax tree from the token and returns a list
 * of all the ancestors of the token. The array indexes are
 * represented as `[0-9]`.
 */
function getAncestryOfToken(token, document) {
    // If we are at the property name, we ignore it to correctly
    // find the parent.
    const isAutocompletingPropertyName = ['ObjectExpression', 'PropertyDefinition'].includes(token.name) ||
        (token.parent?.parent?.name === 'ObjectExpression' && !token.prevSibling);
    // We reverse the list as we want to start from the root.
    const list = getAncestorList(isAutocompletingPropertyName ? token.parent?.parent : token, document).reverse();
    return list;
}
exports.getAncestryOfToken = getAncestryOfToken;
//# sourceMappingURL=utils.js.map